@@ -12,7 +12,7 @@ import zipfile
 logger = logging.getLogger(__name__)
 
 
-def copytree(src, dst, symlinks=0):
+def copytree(src, dst, symlinks=0, allowed_basenames=None):
     """Local implementation of shutil's copytree function.
 
     Checks wheather destination directory exists or not
@@ -27,12 +27,17 @@ def copytree(src, dst, symlinks=0):
             continue
         srcname = os.path.join(src, name)
         dstname = os.path.join(dst, name)
+        if allowed_basenames:
+            if os.path.isfile(srcname):
+                if name not in allowed_basenames:
+                    logger.debug("Skipped %s" % srcname) 
+                    continue
         try:
             if symlinks and os.path.islink(srcname):
                 linkto = os.readlink(srcname)
                 os.symlink(linkto, dstname)
             elif os.path.isdir(srcname):
-                copytree(srcname, dstname, symlinks)
+                copytree(srcname, dstname, symlinks, allowed_basenames)
             elif not os.path.isfile(dstname) and symlinks:
                 os.symlink(srcname, dstname)
             elif not symlinks:
@@ -140,7 +145,7 @@ class Recipe(zc.recipe.egg.Eggs):
 
     def install_appengine(self):
         """Downloads and installs Google App Engine."""
-        arch_filename = self.options['url'].split('/')[-1]
+        arch_filename = self.options['url'].split(os.sep)[-1]
         dst = os.path.join(self.buildout['buildout']['parts-directory'])
         downloads_dir = os.path.join(os.getcwd(), 'downloads')
         if not os.path.isdir(downloads_dir):
@@ -155,7 +160,7 @@ class Recipe(zc.recipe.egg.Eggs):
             shutil.rmtree(os.path.join(dst, 'google_appengine'))
         arch = zipfile.ZipFile(open(src, "rb"))
         for name in arch.namelist():
-            if name.endswith('/'):
+            if name.endswith(os.sep):
                 os.mkdir(os.path.join(dst, name))
             else:
                 outfile = open(os.path.join(dst, name), 'wb')
@@ -195,9 +200,15 @@ class Recipe(zc.recipe.egg.Eggs):
         for key in entries.keys():
             src = os.path.join(entries[key], key.split('.')[0])
             dir = os.path.join(lib, os.path.basename(src))
+            egg_info_src = os.path.join(ws.by_key[key]._provider.egg_info,
+                                        'SOURCES.txt')
+            sources = open(egg_info_src, 'r')
+            allowed_basenames = [os.path.basename(p.strip())
+                                 for p in sources.readlines()]
+            sources.close()
             if not os.path.exists(dir):
                 os.mkdir(dir)
-            copytree(src, dir, True)
+            copytree(src, dir, True, allowed_basenames=allowed_basenames)
 
     def zip_packages(self, ws, lib):
         """Creates zip archive of configured packages."""
@@ -209,16 +220,13 @@ class Recipe(zc.recipe.egg.Eggs):
         for root, dirs, files in os.walk('.'):
             for f in files:
                 ext = os.path.splitext(f)[1]
-                if ext not in ('.pyc', '.pyo', '.so', '.h', '.c') \
-                and f.find('coptimizations') == -1:
+                if ext not in ('.h', '.c'):
                     zipper.add(os.path.join(root, f))
         pkg_resources_dummy = open(os.path.join(lib, 'pkg_resources.py'), "w")
         pkg_resources_dummy.write("def _dummy_func(*args):\n")
         pkg_resources_dummy.write("    pass\n\n")
         pkg_resources_dummy.write("declare_namespace = _dummy_func\n")
         pkg_resources_dummy.write("resource_filename = _dummy_func\n")
-        pkg_resources_dummy.write("import imp\n\n")
-        pkg_resources_dummy.write("imp.load_dynamic = _dummy_func\n\n")
         pkg_resources_dummy.close()
         zipper.add('pkg_resources.py')
         zipper.close()
